<?php

namespace Litecms\Masterplan\Policies;

use Litepie\User\Contracts\UserPolicy;
use Litecms\Masterplan\Models\MasterPlan;

class MasterPlanPolicy
{

    /**
     * Determine if the given user can view the master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function view(UserPolicy $user, MasterPlan $master_plan)
    {
        if ($user->canDo('masterplan.master_plan.view') && $user->isAdmin()) {
            return true;
        }

        return $master_plan->user_id == user_id() && $master_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can create a master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('masterplan.master_plan.create');
    }

    /**
     * Determine if the given user can update the given master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function update(UserPolicy $user, MasterPlan $master_plan)
    {
        if ($user->canDo('masterplan.master_plan.edit') && $user->isAdmin()) {
            return true;
        }

        return $master_plan->user_id == user_id() && $master_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, MasterPlan $master_plan)
    {
        return $master_plan->user_id == user_id() && $master_plan->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function verify(UserPolicy $user, MasterPlan $master_plan)
    {
        if ($user->canDo('masterplan.master_plan.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given master_plan.
     *
     * @param UserPolicy $user
     * @param MasterPlan $master_plan
     *
     * @return bool
     */
    public function approve(UserPolicy $user, MasterPlan $master_plan)
    {
        if ($user->canDo('masterplan.master_plan.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
