<?php

namespace Litecms\Masterplan\Providers;

use Illuminate\Support\ServiceProvider;

class MasterplanServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'masterplan');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'masterplan');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerMasterplan();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerMasterplan()
    {
        $this->app->bind('masterplan', function($app) {
            return new Masterplan($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Masterplan', 'Lavalite\Masterplan\Facades\Masterplan');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litecms.masterplan', function ($app) {
            return $this->app->make('Litecms\Masterplan\Masterplan');
        });

                // Bind MasterPlan to repository
        $this->app->bind(
            'Litecms\Masterplan\Interfaces\MasterPlanRepositoryInterface',
            \Litecms\Masterplan\Repositories\Eloquent\MasterPlanRepository::class
        );

        $this->app->register(\Litecms\Masterplan\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Litecms\Masterplan\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and masterplan's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/masterplan.php', 'masterplan'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeMasterplan::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litecms.masterplan'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litecms/masterplan.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/masterplan')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/masterplan')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
